# [START app]
import logging
import os
import json
import sqlite3

from flask import Flask, request, make_response, abort, send_file, jsonify, send_from_directory, render_template_string, render_template, g
from flask_httpauth import HTTPBasicAuth
from flask_cors import CORS, cross_origin

# [START config]
DATABASE = "/bin/cyberrange/website38/sql_database.db"
app = Flask(__name__, static_folder='/bin/cyberrange/website38', static_url_path='', template_folder='/bin/cyberrange/website38/sql/templates')
app.config['JSONIFY_PRETTYPRINT_REGULAR'] = False
CORS(app)
# [END config]

# [START CTF-WEB-EN-03 : Robots exclusion protocol]
@app.route('/robots.txt', methods=['GET'] )
def getRobot():
    return app.send_static_file("robots/robots.txt")

@app.route('/secret.html', methods=['GET'] )
def getSecret():
    return app.send_static_file("robots/secret.html")
# [END CTF-WEB-EN-03 : Robots exclusion protocol]

# [START CTF-WEB-EN-04 : Javascript]
@app.route('/q4', methods=['GET'] )
def quest4_1():
    return app.send_static_file("q4/login.html")

@app.route('/q4', methods=['POST'] )
def quest4_2():
    user = request.form.get('user')
    pw = request.form.get('pw')
    if user == "root" and pw == "JavaScriptIsSecure":
        return jsonify({'result': 'flag{Flawed JavaScript}'})
    return jsonify({'result': 'wrong'}) 
# [END CTF-WEB-EN-04 : Javascript]

# [START CTF-WEB-EN-05 : HTTP Header]
@app.route('/q5', methods=['GET'] )
def quest5_1():
    return app.send_static_file("q5/login.html")

@app.route('/q5', methods=['POST'] )
def quest5_2():
    auth_temp = request.authorization
    if auth_temp and auth_temp.username == "root" and auth_temp.password == "toor":
        return jsonify({'result': 'flag{Authorization Header}'})
    return jsonify({'result': 'wrong'})
# [END CTF-WEB-EN-05 : HTTP Header]

#[START CTF-WEB-EN-06 : Python injection]
@app.route('/q6')
def hello_ssti():
    person = [{'name':"Kotona", 'password':"cGFzc3dvcmQ="},
                {'name':"David", 'password':"enhjdmJubQ=="},
                {'name':"John", 'password':"cWF6d3N4ZWRj"},
                {'name':"root", 'password':"ZmxhZ3tUZW1wbGF0ZSBJbmplY3Rpb259"}]

    if request.args.get('name'):
        for i in range(len(person)):
            if person[i]['name'] == request.args.get('name'):
                template = '''<div style="padding: 60px;text-align: center; font-size: 24px;">
                                <h1>CyTrONE Web CTF</h1>
                                <p>Question 6</p>
                            </div>
                            <div style="text-align: center;font-size: 18px;">
                                <form action="" method="get">
                                    Name: <input type="text" name="name"><br>
                                    <input style="margin-top:10px;margin-bottom: 10px;" type="submit" value="Submit">
                                </form>
                                <h2>Hello %s!</h2>
                                <h2>You are a member!</h2>
                            </div>''' % person[i]['name']
                return render_template_string(template, person=person)
            else:
                template = '''<div style="padding: 60px;text-align: center; font-size: 24px;">
                                <h1>CyTrONE Web CTF</h1>
                                <p>Question 6</p>
                            </div>
                            <div style="text-align: center;font-size: 18px;">
                                <form action="" method="get">
                                    Name: <input type="text" name="name"><br>
                                    <input style="margin-top:10px;margin-bottom: 10px;" type="submit" value="Submit">
                                </form>
                                <h2>Hello %s!</h2>
                                <h2>You are not a member!</h2>
                            </div>''' % request.args.get('name')
    else: template = '''<div style="padding: 60px;text-align: center; font-size: 24px;">
                                <h1>CyTrONE Web CTF</h1>
                                <p>Question 6</p>
                            </div>
                            <div style="text-align: center;font-size: 18px;">
                                <form action="" method="get">
                                Name: <input type="text" name="name"><br>
                                <input style="margin-top:10px;margin-bottom: 10px;" type="submit" value="Submit">
                            </form></div>'''
    return render_template_string(template, person=person)
#[END CTF-WEB-EN-06 : Python injection]

#[START CTF-WEB-EN-07 : SQL injection]
def get_db():
    if os.path.exists(DATABASE):
        os.remove(DATABASE)
    conn = sqlite3.connect(DATABASE)
    cur = conn.cursor()
    cur.execute("CREATE TABLE users (fname TEXT, lname TEXT, age INTEGER);")
    cur.execute("CREATE TABLE secret (fname TEXT, lname TEXT, age INTEGER);")
    conn.commit()
    cur.execute("INSERT INTO users VALUES('Mike', 'Tyson', '20');")
    cur.execute("INSERT INTO users VALUES('Thomas', 'Jasper', '30');")
    cur.execute("INSERT INTO users VALUES('Jerry', 'Mouse', '40');")
    cur.execute("INSERT INTO users VALUES('Peter', 'Pan', '50');") 
    cur.execute("INSERT INTO secret VALUES('flag{SQLi}', '', '');")
    conn.commit()
    conn.close()

    db = getattr(g, '_database', None)
    if db is None:
        db = g._database = sqlite3.connect(DATABASE)
    return db

@app.teardown_appcontext
def close_connection(exception):
    db = getattr(g, '_database', None)
    if db is not None:
        db.close()

@app.route('/q7', methods=['GET'])
def sql():
    cur = get_db().cursor()
    if request.args.get('query'):
        res = cur.execute("select * from users where fname='%s'" % request.args.get('query'))
        return render_template("index.html", users=res)
    else:
        template = '''<div style="padding: 60px;text-align: center; font-size: 24px;">
            <h1>CyTrONE Web CTF</h1>
            <p>Question 7</p>
        </div> 
        <div style="text-align: center;font-size: 18px;">
            <form action="" method="get">
                First Name: <input type="text" name="query"><br>
                <input style="margin-top:10px;margin-bottom: 10px;" type="submit" value="Submit">
            </form>
        </div>'''
        return render_template_string(template)
#[END CTF-WEB-EN-007 : SQL injection]


# [START Handling error]
@app.errorhandler(500)
def server_error(e):
    return make_response(jsonify({'error': 'An error occurred during a request'}), 500)

@app.errorhandler(400)
def bad_request(error):
    return make_response(jsonify({'error': 'Bad request'}), 400)

@app.errorhandler(404)
def not_found(error):
    return make_response(jsonify({'error': 'Not found'}), 404)
# [END Handling error]

if __name__ == '__main__':
    app.run(host='127.0.0.1', port=8080, debug=False)
# [END app]
